#import <Foundation/Foundation.h>

#define ToRad(deg) 		( (M_PI * (deg)) / 180.0f )
#define ToDeg(rad)		( (180.0f * (rad)) / M_PI )
#define SQR(x)			( (x) * (x) )
#define MINUTES_PER_CARDINAL_DIRECTION (360.0f / 8.0f)


/** Given the vector v from p1 to p2, computes the angle in degrees between v and the compared vector <0,1> (<0,-1> if flipped).
 * The returned angle is in the range [0,360]). */
static inline CGFloat AngleFromNorth(NSPoint p1, NSPoint p2, BOOL flipped) {
    NSPoint v = NSMakePoint(p2.x-p1.x,p2.y-p1.y);
    CGFloat vmag = (CGFloat)sqrt(SQR(v.x) + SQR(v.y)), result = 0;
    v.x /= vmag;
    v.y /= vmag;
    result = (CGFloat)ToDeg((CGFloat)atan2(v.x,(flipped ? -v.y : v.y)));
    return (result >=0  ? result : result + 360.0f);
}

/** Return the midpoint of a NSRect */
static inline NSPoint KRRectMidpt(NSRect rect) {
	return NSMakePoint(NSMidX(rect), NSMidY(rect));
	
}

/** Return an NSRect centered at the midpoint with size */
static inline NSRect KRMakeRectWithMidpointAndSize(NSPoint point, NSSize size) {
	NSRect rect = NSMakeRect(point.x - (size.width/2), point.y - (size.height/2), size.width, size.height);
	
	return rect;
	
}

/** Returns YES if the point is in the rectangle. */
static inline BOOL PointIsInView(NSPoint locationInWindow, NSRect frame) {
    BOOL hit = NO;
    if ( NSPointInRect(locationInWindow, frame) ) {
		CGFloat radius = (MIN(NSHeight(frame), NSWidth(frame)) / 2.0f);
		NSPoint centerPoint = NSMakePoint((CGFloat)floor(NSMinX(frame)+radius+.5f), (CGFloat)floor(NSMinY(frame)+radius+.5f));
		CGFloat sqDistanceToCenter = SQR(locationInWindow.x- centerPoint.x) + SQR(locationInWindow.y- centerPoint.y);
		hit = sqDistanceToCenter < SQR(((9.0/10.0) * radius));
    }
    return hit;
}


#pragma mark -
#pragma mark Scale Around Midpoint
/** Scales the current transofmration matrix centered at the midpoint. */
static inline void KRScaleCTMAroundMidpoint(CGFloat scaleFactor, 
											NSPoint midpoint) {
	NSAffineTransform *scaleTransform = [NSAffineTransform transform];
	[scaleTransform scaleBy:scaleFactor];
	/* compute the distance that the scale moves the transformed point.  */
	NSPoint scaledPoint = [scaleTransform transformPoint:midpoint];
	NSPoint delta = NSMakePoint(midpoint.x-scaledPoint.x, midpoint.y-scaledPoint.y);
	NSAffineTransform *deltaCompensation = [NSAffineTransform transform];
	[deltaCompensation translateXBy:delta.x
								yBy:delta.y];
	[deltaCompensation concat];
	[scaleTransform concat];
	
}


/** Return an NSAffineTransform which rotates in degrees counterclockwise around a midpoint. */
static inline NSAffineTransform *KRMakeTransformScaleAroundMidpoint(CGFloat scaleFactor, 
																	NSPoint midpoint) {
	NSAffineTransform *scaleTransform = [NSAffineTransform transform];
	[scaleTransform scaleBy: scaleFactor];
	/* compute the distance that the rotation moves the transformed point.  */
	NSPoint scaledPoint = [scaleTransform transformPoint:midpoint];
	NSPoint delta = NSMakePoint(midpoint.x-scaledPoint.x, midpoint.y-scaledPoint.y);
	NSAffineTransform *deltaCompensation = [NSAffineTransform transform];
	[deltaCompensation translateXBy:delta.x
								yBy:delta.y];
	NSAffineTransform *compositeTransform = [NSAffineTransform transform];
	[compositeTransform prependTransform:deltaCompensation];
	[compositeTransform prependTransform:scaleTransform];
	return compositeTransform;
}

#pragma mark -
#pragma mark Rotate Around Midpoint
/** Rotates the current transofmration matrix in degrees counterclockwise around a midpoint. */
static inline void KRRotateCTMAroundMidpoint(CGFloat degreesToRotate, 
											 NSPoint midpoint) {
	NSAffineTransform *rotationTransform = [NSAffineTransform transform];
	[rotationTransform rotateByDegrees:degreesToRotate];
	/* compute the distance that the rotation moves the transformed point.  */
	NSPoint rotatedPoint = [rotationTransform transformPoint:midpoint];
	NSPoint delta = NSMakePoint(midpoint.x-rotatedPoint.x, midpoint.y-rotatedPoint.y);
	NSAffineTransform *deltaCompensation = [NSAffineTransform transform];
	[deltaCompensation translateXBy:delta.x
								yBy:delta.y];
	[deltaCompensation concat];
	[rotationTransform concat];
	
}




/** Return an NSAffineTransform which rotates in degrees counterclockwise around a midpoint. */
static inline NSAffineTransform *KRMakeTransformRotateAroundMidpoint(CGFloat degreesToRotate, 
																	 NSPoint midpoint) {
	NSAffineTransform *rotationTransform = [NSAffineTransform transform];
	[rotationTransform rotateByDegrees:degreesToRotate];
	/* compute the distance that the rotation moves the transformed point.  */
	NSPoint rotatedPoint = [rotationTransform transformPoint:midpoint];
	NSPoint delta = NSMakePoint(midpoint.x-rotatedPoint.x, midpoint.y-rotatedPoint.y);
	NSAffineTransform *deltaCompensation = [NSAffineTransform transform];
	[deltaCompensation translateXBy:delta.x
								yBy:delta.y];
	NSAffineTransform *compositeTransform = [NSAffineTransform transform];
	[compositeTransform prependTransform:deltaCompensation];
	[compositeTransform prependTransform:rotationTransform];
	return compositeTransform;
}


#pragma mark -
#pragma mark Simple Convenience
static inline CGFloat KRLesserFloat(CGFloat x, CGFloat y) {
	if (x <= y) { return x; }
	else { return y; }
}
static inline CGFloat KRGreaterFloat(CGFloat x, CGFloat y) {
	if (x >= y) { return x; }
	else { return y; }
}


static inline CGFloat KRLesserDimensionOfRect(NSRect rect) {
	CGFloat height = rect.size.height;
	CGFloat width = rect.size.width;
	if (width > height) { return height; }
	else { return width; }
}

static inline CGFloat KRGreaterDimensionOfRect(NSRect rect) {
	CGFloat height = rect.size.height;
	CGFloat width = rect.size.width;
	if (width > height) { return width; }
	else { return height; }
}

static inline void KRRectStandardize(NSRect *drawingRect_p) {
	CGRect tempRect = NSRectToCGRect(*drawingRect_p);
	tempRect = CGRectStandardize(tempRect);
	*drawingRect_p = NSRectFromCGRect(tempRect);
}


static inline CGFloat KRGreatestSizeValue(NSSize size) {
	if (size.height <= size.width) { return size.width; }
	else { return size.height; }
}

static inline CGFloat KRSmallestSizeValue(NSSize size) {
	if (size.height <= size.width) { return size.height; }
	else { return size.width; }
}


static inline NSRect KRRectCenteredInRect(NSRect rect) {
	/* Setup the inset rectangles to draw into (centered in the rect) */
	CGFloat lesserDimensionOfRect = KRLesserDimensionOfRect(rect);
	NSRect centeredRect = NSInsetRect(rect, (rect.size.width-lesserDimensionOfRect) /2, (rect.size.height-lesserDimensionOfRect)/2);
	
	return centeredRect;
}
